import json
import os
import requests

BASE_URL = os.environ.get('BOBO_URL', 'https://api.bobolink.me/v1/')

def raise_bad_request(request, parse_json=True):
    try:
        request.raise_for_status()
        return json.loads(request.text) if parse_json else request.text
    except requests.exceptions.HTTPError as e:
        raise Exception(request.text) from e    


def signup(email, password):
    r = requests.post(BASE_URL + 'users', json={'email':email, 'password':password})    
    return raise_bad_request(r, parse_json=False)


def reset_password(email, new_password, token):
    r = requests.put(BASE_URL + 'users/account/password',
                         json={'email':email, 'new':new_password, 'token':token})
    return raise_bad_request(r, parse_json=False)


def send_reset_token(email):
    r = requests.put(BASE_URL + 'users/account/password', json={'email':email})
    return raise_bad_request(r, parse_json=False)


def get_token(email, password):
    r = requests.get(BASE_URL + 'token', auth=(email, password))
    return raise_bad_request(r, parse_json=False)


def add_bookmarks(creds, bookmarks):
    r = requests.post(BASE_URL + 'bookmarks', auth=(creds['email'], creds['token']), 
                      json={'urls': bookmarks})
    return raise_bad_request(r)


def delete_bookmarks(creds, bookmarks):
    r = requests.delete(BASE_URL + 'bookmarks', auth=(creds['email'], creds['token']),
                      json={'urls': bookmarks})
    return raise_bad_request(r)


def get_user_data(creds):
    email = creds['email']
    r = requests.post(BASE_URL + 'users/search', auth=(email, creds['token']),
                      json={'email': email})
    return raise_bad_request(r)


def get_user_bookmarks(creds):
    user = get_user_data(creds)
    r = requests.get(f'{BASE_URL}users/{user["id"]}/bookmarks',
                     auth=(creds['email'], creds['token']))
    return raise_bad_request(r)


def search_bookmarks(creds, query, field=None):
    if field is None:
        field = 'content'
    r = requests.post(BASE_URL + 'bookmarks/search', auth=(creds['email'], creds['token']),
                      json={'query': query, 'field': field})
    return raise_bad_request(r)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bobolink']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0', 'requests>=2.25.1,<3.0.0', 'termcolor>=1.1.0,<2.0.0']

entry_points = \
{'console_scripts': ['bobolink = bobolink.main:cli']}

setup_kwargs = {
    'name': 'bobolink',
    'version': '0.1.0',
    'description': 'bobolink stores bookmarks and helps you search for them later',
    'long_description': '# bobolink-cli\n\nbobolink is a small tool that helps user\'s store bookmarks and search for them later.\nMore specifically, bobolink provides full text search on the HTML documents of the bookmarks that you\'ve saved. In practice, this allows you to use bobolink to save some links, and later search/return all your links which contain for example `"(songbird OR blackbird) AND NOT currawong".`\n\nFor more information on bobolink in general, please refer to the [website](https://bobolink.me)\n\nbobolink-cli is the command line interface to the public instance \nof the bobolink [backend.](https://github.com/jtanza/bobolink/)\n\n### Installation\n\n```\n$ python -m pip install bobolink\n```\n\n### Getting Started\n\nFor user\'s without a bobolink account, the fastest way to get going is to run\n`bobolink signup` after installation. This, followed by `bobolink configure` is all that is needed in order to start saving and searching your bookmarks.\n\nThe cli is heavily documented and can be accessed at anytime directly via \n`bobolink [COMMAND] --help`. Please refer to the terminal session below for an exploration\nof what\'s possible with bobolink.\n',
    'author': 'jtanza',
    'author_email': 'tanzajohn@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jtanza/bobolink-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import click
import configparser
from bobolink import api
from pathlib import Path
from termcolor import colored

INI_PATH = str(Path.home()) + '/.bobolink'

@click.group()
@click.version_option("0.1.0")
def cli():
  '''Bobolink - dump links, search them later.'''
  pass


@cli.command()
@click.option('--email', prompt=True)
@click.option('--password', prompt=True, hide_input=True,
              confirmation_prompt=True)
def signup(email, password):
  '''Creates a new bobolink user.'''
  try:
    api.signup(email, password)
    click.echo('Success! Your account has succesfully been created.\n'
    'Please run [bobolink configure] to initialize your environment.')
  except Exception as e:    
    click.echo(f'Error while creating user: {e}')


@cli.command()
@click.option('--email', prompt=True)
@click.option('--password', prompt=True, hide_input=True)
def configure(email, password):
  '''Creates a bobolink credentials file.

  Creates an INI file containing a user's email and API
  token which are used to authenticate all requests made
  from the bobolink CLI.

  Users wishing to refresh their API tokens can simply rerun
  [bobolink configure] at any time.
  '''
  try:
    token = api.get_token(email, password)
    
    Path(INI_PATH).mkdir(exist_ok=True)

    config = configparser.ConfigParser()
    config['default'] = {'bobolink_email': email,
                         'bobolink_token': token}

    with open(f'{INI_PATH}/credentials', 'w') as f:
      config.write(f)

    click.echo(f'Success! Configuration written to {INI_PATH}.\nBobolink is ready to use.')
  except Exception as e:
    click.echo(f'Error while configuring environment: {e}')


def get_creds():
  '''Reads user credentials from INI file'''
  config = configparser.ConfigParser()
  config.read(f'{INI_PATH}/credentials')

  default = config['default']
  return {'token': default['bobolink_token'],
          'email': default['bobolink_email']}


@cli.command()
@click.option('--get-reset-token', is_flag=True,
              help='Send the requisite reset token to the user\'s email')
def reset_password(get_reset_token):
  '''Resets a users bobolink password.

  User's should first run with [--get-reset-token] in order to generate
  a reset token (required in order to perform a password reset).

  Authentication tokens are invalidated after a password reset. User's
  should run [bobolink configure] in order to generate a new, valid auth token.
  '''
  email = get_creds()['email']
  try:
    if get_reset_token:
      api.send_reset_token(email)
      click.echo(f'Success! Reset token sent to {email}')
    else:
      reset_token = click.prompt('Token', hide_input=True)
      new_password = click.prompt('New Password', hide_input=True, confirmation_prompt=True)
      api.reset_password(email, new_password, reset_token)
      click.echo('Success! Password has been reset.\n'
                'Be sure to run [bobolink configure] to update'
                'your environment with your new password.')
  except Exception as e:
    click.echo(f'Error while resetting password: {e}')


@cli.command()
@click.argument('bookmarks', nargs=-1)
def add(bookmarks):
  '''Adds new bookmarks to a user's store.

  BOOKMARKS should be a whitespace seperated list of
  URLs.

  Bookmarks added here are immediately made searchable
  via [bobolink search].

  Users wishing to refresh the searchable content of a
  bookmark can simply rerun this command with the bookmark URL.
  '''
  try:
    added = api.add_bookmarks(get_creds(), bookmarks)
    click.echo('Success! Added:\n' + '\n'.join(added))
  except Exception as e:
    click.echo(f'Error while adding bookmarks: {e}')


@cli.command()
@click.argument('bookmarks', nargs=-1)
def delete(bookmarks):
  '''Deletes a bookmark from a user's store.

  BOOKMARKS should be a whitespace seperated list of
  URLs.
  '''
  try:
    deleted = api.delete_bookmarks(get_creds(), bookmarks)
    click.echo('Success! Deleted:\n' + '\n'.join(deleted))
  except Exception as e:
    click.echo(f'Error while deleting bookmarks: {e}')


@cli.command()
def export():
  '''Provides a url dump of all a user's stored bookmarks'''
  try:
    bookmarks = api.get_user_bookmarks(get_creds())
    click.echo('\n'.join(bookmarks))
  except Exception as e:
    click.echo(f'Error while exporting bookmarks: {e}')


def format_hit(hit, url_only, field):
  url = f'{colored("[url]", "green")}: {colored(hit["url"], attrs=["underline"])}'

  if url_only or field == "url":
    return url

  arr = []
  for word in hit['content'].split(' '):
    if word.startswith('<b>'):
      word = colored(word.replace('<b>', '').replace('</b>', ''), 'yellow')
    arr.append(word)

  match = f'{colored("[match]", "green")}: {" ".join(arr)}'
  return f'{url}\n{match}'


@cli.command()
@click.argument('query')
@click.option('--field', type=click.Choice(['url', 'content']),
              help='the bookmark field to search on')
@click.option('--url-only', is_flag=True, help='only return the URL of a match')
def search(query, field, url_only):
  '''Searches a query string against a user's bookmark store.

  When no field is supplied, searches the text content of the HTML
  page associated with a user's bookmark.

  When searching on content, an attempt is made to highlight where those
  matches occured in the original HTML. Support for this feature varies
  by terminal.

  Search syntax for bobolink is equivalent to the query language supported
  by Lucene's Query Parser. User's should refer to the Lucene documentation
  for more info and example usage: https://lucene.apache.org/core/2_9_4/queryparsersyntax.html
  '''
  try:
    hits = api.search_bookmarks(get_creds(), query, field)
    formatted = list(map(lambda hit: format_hit(hit, url_only, field), hits))
    click.echo('\n'.join(formatted))
  except Exception as e:
    click.echo(f'Error while searching bookmarks: {e}')
  

if __name__ == "__main__":
  cli()
  